package org.netkernel.wiki.rm.endpoint;

import java.net.URI;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class ImageRenderEngine extends StandardAccessorImpl
{
	public static final String SVG_NAMESPACE="http://www.w3.org/2000/svg";
	
	public ImageRenderEngine()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext aContext) throws Exception
	{	String type=aContext.getThisRequest().getArgumentValue("activeType");
		String base=(String)aContext.source("arg:base");
		String opd=(String)aContext.source("arg:operand", String.class);
		opd=opd.replaceAll("[\\r\\f\n]", "");
		int idx=Integer.parseInt(aContext.getThisRequest().getArgumentValue("index").substring(2));
		boolean svg=opd.contains(SVG_NAMESPACE);
		Object result=null;
		if(type.endsWith("XHTML"))
		{	//SVG rendering not implemented yet
			if(svg)
			{	//result=opd;
				URI opduri=URI.create(aContext.getThisRequest().getArgumentValue("entry"));
				URI baseuri=URI.create("res:/wiki/");
				opduri=baseuri.relativize(opduri);
				//Have to use Javascript to detect IE and let it use a reference to a PNG renderer - oh Bill, running faster beats tying shoe laces together.
				StringBuilder sb=new StringBuilder(256);
				opd=opd.replaceAll("\"","\\\\\"");
				opd=XMLUtils.escape(opd);
				sb.append("\n<div id=\"image"+idx+"\"/>");  //A block to write image into
				//A script that only a proper XHTML browser can run
				sb.append("\n<script language='javascript' type='text/javascript'>");
				sb.append("\ntry{");
				sb.append("\nvar parser = new DOMParser();");   //Fail fast
				sb.append("\nvar img=\""+opd+"\"; ");
				sb.append("\nvar target=document.getElementById(\"image"+idx+"\");");
				sb.append("\nvar e = parser.parseFromString(img, 'text/xml');");
				sb.append("\ntarget.appendChild(e.documentElement);");
				sb.append("\n}catch(ex){}; //Fooled IE! Ha Ha");
				sb.append("\n</script>");
				//A script in an XML comment that only a non XHTML browser, ie crappy IE, will execute
				sb.append("\n<script language='javascript' type='text/javascript'>");
				sb.append("\n<!--");
				sb.append("\nvar img=\"<img xmlns=\\\"http://www.w3.org/1999/xhtml\\\" src=\\\""+base+"/svgs/"+opduri.toString()+"+index@i:"+idx+"\\\"/>\";");
				sb.append("\nvar target=document.getElementById(\"image"+idx+"\");");
				sb.append("\ntarget.innerHTML=img;");
				sb.append("\n--></script>");
				result=sb.toString();
				//xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
				//xmlDoc.async="false";
				//xmlDoc.loadXML(txt);
			}
			else
			{	result="<img src=\""+opd+"\"/>";
			}
		}
		else
		{	//Docbook
			StringBuilder sb=new StringBuilder(512);
			sb.append("<mediaobject><imageobject>");
			if(svg)
			{	sb.append(opd);
			}
			else
			{	sb.append("<imagedata fileref=\""+opd+"\"/>");
			}
			sb.append("</imageobject></mediaobject>");
			result=sb.toString();		
		}
		aContext.createResponseFrom(result);
	}
}
