package org.netkernel.wiki.rm.endpoint;

import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.mylyn.wikitext.confluence.core.ConfluenceLanguage;
import org.eclipse.mylyn.wikitext.core.parser.DocumentBuilder;
import org.eclipse.mylyn.wikitext.core.parser.MarkupParser;
import org.eclipse.mylyn.wikitext.core.parser.builder.DocBookDocumentBuilder;
import org.eclipse.mylyn.wikitext.core.parser.builder.HtmlDocumentBuilder;
import org.eclipse.mylyn.wikitext.core.parser.markup.MarkupLanguage;
import org.eclipse.mylyn.wikitext.markdown.core.MarkdownLanguage;
import org.eclipse.mylyn.wikitext.mediawiki.core.MediaWikiLanguage;
import org.eclipse.mylyn.wikitext.textile.core.TextileLanguage;
import org.eclipse.mylyn.wikitext.tracwiki.core.TracWikiLanguage;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class TextileJParserEndpoint extends StandardAccessorImpl
{
	private static final String XHTML_MODE="XHTML";
	private static final String DOCBOOK_MODE="DOCBOOK";
	
	private static Pattern sHTTPHack=Pattern.compile("http:/([^/])");
		
	public TextileJParserEndpoint()
	{	this.declareThreadSafe();
	}

	@Override
	public void onSource(INKFRequestContext aContext) throws Exception
	{	String mode=aContext.getThisRequest().getArgumentValue("mode");
		String opd=(String)aContext.source("arg:operand", String.class);
		
		URI base=null;
		if(aContext.getThisRequest().argumentExists("base"))
		{	base=URI.create(aContext.getThisRequest().getArgumentValue("base"));
			if(base.isAbsolute())
			{	base=URI.create(base.getPath());				
			}
		}
		
		//Syntax
		String syntax=aContext.getThisRequest().getArgumentValue("syntax");
		MarkupLanguage lang=null;
		if(syntax.equals("TEXTILE"))
		{	lang=new TextileLanguage();
		}
		else if(syntax.equals("MEDIAWIKI"))
		{	lang=new MediaWikiLanguage();
		}
		else if(syntax.equals("TRACWIKI"))
		{	lang=new TracWikiLanguage();			
		}
		else if(syntax.equals("CONFLUENCE"))
		{	lang=new ConfluenceLanguage();
		}
		else if(syntax.equals("MARKDOWN"))
		{	lang=new MarkdownLanguage();
		}
		
		//Prepare Parser Engine
		MarkupParser parser=new MarkupParser(lang);
		ByteArrayOutputStream baos=new ByteArrayOutputStream(2048);
		DocumentBuilder builder=null;
		OutputStreamWriter writer=new OutputStreamWriter(baos);
		boolean xhtml=true;
		if(mode.equals(XHTML_MODE))
		{	HtmlDocumentBuilder localbuilder=new HtmlDocumentBuilder(writer);
			localbuilder.setEmitAsDocument(false);
			baos.write("<div>".getBytes());	
			if(base!=null)
			{	localbuilder.setBase(base);
			}
			builder=localbuilder;
		}
		else if(mode.equals(DOCBOOK_MODE))
		{	xhtml=false;
			DocBookDocumentBuilder localbuilder=new DocBookDocumentBuilder(writer);
			if(base!=null)
			{	localbuilder.setBase(base);
			}
			builder=localbuilder;
		}
		else
		{	throw new Exception("Unsupported mode: "+mode);			
		}

		//Main Parse
		parser.setBuilder(builder);		
		parser.parse(opd);
		writer.close();
		if(xhtml)
		{	baos.write("</div>".getBytes());	
		}
		baos.close();
		opd=baos.toString();
		
		// fix to work around inability to put host relative links into wiki text
		// if you put in http:/a/b it will be replaced with /a/b
		Matcher m=sHTTPHack.matcher(opd);
		if(m.find())
		{
			StringBuffer sb=new StringBuffer(opd.length());
			do
			{	m.appendReplacement(sb, "/$1");
			} while (m.find());
			m.appendTail(sb);
			opd=sb.toString();
		}
		
		INKFResponse resp=aContext.createResponseFrom(opd);
		resp.setMimeType("text/xml");
	}
	
}
